unit uPSDBTreeView.MC;

interface

uses
  System.SysUtils, System.Classes, FireDAC.Stan.Intf, FireDAC.Stan.Option, FireDAC.Stan.Error, FireDAC.UI.Intf,
  FireDAC.Phys.Intf, FireDAC.Stan.Def, FireDAC.Stan.Pool, FireDAC.Stan.Async, FireDAC.Phys, FireDAC.VCLUI.Wait, Data.DB,
  FireDAC.Comp.Client,

  // my
  uPSDBTreeView.MainFrame,
  uPSDBTreeView.Update,
  uPSDBTreeView.SQL,
  uPSDBTreeView.Delete,
  uPSDBTreeView.CommonMethods,
  uPSDBTreeView.Checkboxes,
  uPSDBTreeView.GetSelectedElements,
  uPSDBTreeView.Add,
  uPSDBTreeView.NodeUpDown,
  uPSDBTreeView.CopyCutPaste;



type TDBFields=class (TComponent)

 private

    Fid:string;
    FidParent:string;
    Fname:string;
    FsortIndex:string;
    FisFolder:string;
    FisTrash:string;
    FisChecked:string;
    Fexternal_id:string;



 public

     property   id:string read Fid write Fid;
     property   idParent:string read FidParent write FidParent;
     property   name:string read fname write fname;
     property   sortIndex:string read FsortIndex write FsortIndex;
     property   isFolder:string read FisFolder write FisFolder;
     property   isTrash:string read FisTrash write FisTrash;
     property   isChecked:string read FisChecked write FisChecked;
     property   external_id:string read FExternal_id write FExternal_id;

end;


type
  TPSDBTreeViewMC = class(TDataModule)
    FDConnection: TFDConnection;
    procedure DataModuleDestroy(Sender: TObject);
  private
    { Private declarations }

    FDBName:string;
    FDBTable:string;

    FDBFields:TDBFields;
    FIsUseExternalID:boolean;
    Fexternal_id_value:Integer;
    FMF:TPSDBTreeViewMainFrame; // main frame

    //
    FUpdate:TPSDBTreeViewUpdate;
    FSQL:TPSDBTreeViewSQL;
    FDelete:TPSDBTreeViewDelete;
    FCommonMethods:TPSDBTreeViewCommonMethods;
    FAdd:TPSDBTreeViewAdd;
    FNodeUpDown:TPSDBTreeViewNodeUpDown;
    FCopyCutPaste:TPSDBTreeViewCopyCutPaste;

    FisAlreadyExpandedSL:TStringList;
    FisOnlyFoldersInDBTreeView:Boolean;
    FCheckboxes:TPSDBTreeViewCheckboxes;
    FGetSelectedElements:TPSDBTreeViewGetSelectedElements;

    FidCurrentParent:integer;

  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor  destroy; override;

// main methods
    procedure InitialSetup;
    procedure InitialUpdate;
    procedure DeleteCheckedOrSelected;
    procedure AddFolder(AName:string);
    procedure AddFile(AName:string);
    procedure NodeUp;
    procedure NodeDown;
    procedure Copy;
    procedure Cut;
    procedure Paste;
    procedure ClearAllPointers;
    procedure Rename(ANewName:string);

    property DBFields:TDBFields read FDBFields write FDBFields;
    property IsUseExternalID:Boolean read FIsUseExternalID write FIsUseExternalID;
    property External_id_value:Integer read Fexternal_id_value write Fexternal_id_value;
    property DBName:string read FDBName write FDBName;
    property DBTable:string read FDBTable write FDBTable;


    property   MF:TPSDBTreeViewMainFrame read FMF write FMF;
    property   SQL:TPSDBTreeViewSQL read FSQL write FSQL;
    property   Update:TPSDBTreeViewUpdate read FUpdate write FUpdate;
    property   Delete:TPSDBTreeViewDelete read FDelete write FDelete;
    property   CommonMethods:TPSDBTreeViewCommonMethods read FCommonMethods write FCommonMethods;
    property   Checkboxes:TPSDBTreeViewCheckboxes read FCheckboxes write FCheckboxes;
    property   GetSelectedElements:TPSDBTreeViewGetSelectedElements read FGetSelectedElements write FGetSelectedElements;
    property   AddFolderOrFile:TPSDBTreeViewAdd read FAdd write FAdd;
    property   NodeUpDown:TPSDBTreeViewNodeUpDown read FNodeUpDown write FNodeUpDown;
    property   CopyCutPaste:TPSDBTreeViewCopyCutPaste read FCopyCutPaste write  FCopyCutPaste;

    property   isAlreadyExpandedSL:TStringList read FisAlreadyExpandedSL write FisAlreadyExpandedSL;
    property   isOnlyFoldersInDBTreeView:Boolean read FisOnlyFoldersInDBTreeView write FisOnlyFoldersInDBTreeView;
    property   idCurrentParent:integer read FidCurrentParent write FidCurrentParent;

  end;

//var
//  PSDBTreeViewMC: TPSDBTreeViewMC;

implementation

uses uPSDBTreeView.Main, Vcl.Controls;
var  Main_UV:TPSDBTreeView;

{%CLASSGROUP 'Vcl.Controls.TControl'}

{$R *.dfm}

{ TPSDBTreeViewMC }

procedure TPSDBTreeViewMC.AddFile(AName:string);
begin
FAdd.AddFile(AName);
end;

procedure TPSDBTreeViewMC.AddFolder(AName:string);
begin
FAdd.AddFolder(AName);
end;

procedure TPSDBTreeViewMC.ClearAllPointers;
begin
FDelete.ClearAllPointers;
end;

procedure TPSDBTreeViewMC.Copy;
begin
FCopyCutPaste.Copy;
end;

constructor TPSDBTreeViewMC.Create(AOwner: TComponent);
begin
  inherited;

// if (AOwner is TPSDBTreeView) then Main_UV:=(AOwner as TPSDBTreeView);

 FDBFields:=TDBFields.Create(Self);

 FMF:=TPSDBTreeViewMainFrame.Create(Self);
 FMF.Parent:=(Self.Owner as TPSDBTreeView);
 FMF.Align:=alClient;
 FMF.Show;


 // instances
 FUpdate:=TPSDBTreeViewUpdate.Create(Self);
 FSQL:=TPSDBTreeViewSQL.Create(Self);
 FDelete:=TPSDBTreeViewDelete.Create(Self);
 FCommonMethods:=TPSDBTreeViewCommonMethods.Create(Self);
 FCheckboxes:=TPSDBTreeViewCheckboxes.Create(Self);
 FGetSelectedElements:=TPSDBTreeViewGetSelectedElements.Create(Self);
 FAdd:=TPSDBTreeViewAdd.Create(Self);
 FNodeUpDown:=TPSDBTreeViewNodeUpDown.Create(Self);
 FCopyCutPaste:=TPSDBTreeViewCopyCutPaste.Create(Self);

 FisAlreadyExpandedSL:=TStringList.Create;





end;

procedure TPSDBTreeViewMC.Cut;
begin
FCopyCutPaste.Cut;
end;

procedure TPSDBTreeViewMC.DataModuleDestroy(Sender: TObject);
begin
// FCheckboxes.UncheckAllChecked;
end;

procedure TPSDBTreeViewMC.DeleteCheckedOrSelected;
begin
FDelete.DeleteCheckedOrSelected;
end;

destructor TPSDBTreeViewMC.destroy;
begin
FisAlreadyExpandedSL.Free;
  inherited;
end;

procedure TPSDBTreeViewMC.InitialSetup;
begin
//
FSQL.SetDBNamePlusDBTable;
FSQL.SetShortNamesForFields;


end;

procedure TPSDBTreeViewMC.InitialUpdate;
begin

isAlreadyExpandedSL.Clear;
FCheckboxes.ClearAllCheckBoxesInDB;
FCheckboxes.UncheckAllChecked;
FDelete.ClearAllPointers;
mf.TreeView.Items.Clear;

FUpdate.UpdateNodeOnExpanding(nil);


end;

procedure TPSDBTreeViewMC.NodeDown;
begin
FNodeUpDown.NodeDown;
end;

procedure TPSDBTreeViewMC.NodeUp;
begin
FNodeUpDown.NodeUp;
end;

procedure TPSDBTreeViewMC.Paste;
begin
FCopyCutPaste.Paste;
end;

procedure TPSDBTreeViewMC.Rename(ANewName: string);
var idSelected:integer;
begin
if mf.TreeView.Selected=nil then Exit;
idSelected:=integer(mf.TreeView.Selected.Data^);
FSQL.setupdateNameWhereID(idSelected.ToString,ANewName);
FSQL.qUpdateNameWhereID.ExecSQL;
mf.TreeView.Selected.Text:=ANewName;
end;

end.
