unit uPSDBTreeView.Checkboxes;

interface

uses
  System.SysUtils, System.Classes, Vcl.ComCtrls;

type
  TPSDBTreeViewCheckboxes = class(TDataModule)
  private
    { Private declarations }


    FEnableToggleSelectedCheckbox:Boolean;

    FNodeUnderMice:TTreeNode;
    FMouseOnCheckBox:Boolean;
    FUnderNode:Boolean;

    FIDArrayOfAllChildrenRecursive:TArray<integer>;
    FIDArrayOfAllParents:TArray<integer>;

    FMC:TObject;

    procedure CheckNodeAndAllChildren(Node: TTreeNode);
    procedure CheckNodeAndAllVisibleChildren(Node: TTreeNode);
    procedure UnCheckNodeAndAllVisibleChildren(Node: TTreeNode);
    procedure UnCheckNodeAndAllChildren(Node: TTreeNode);
    procedure GatherAllParentsIDAnduncheckInTree(Node: TTreeNode);





  public
    { Public declarations }

    constructor Create(AOwner: TComponent); override;

    procedure ToggleSelectedCheckBox;
    procedure DBTreeViewMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure UncheckAllChecked;
    procedure ClearAllCheckBoxesInDB;

    property NodeUnderMice:TTreeNode read FNodeUnderMice write FNodeUnderMice;
    property IsMiceUnderNode:Boolean read FUnderNode write FUnderNode;
    property MouseOnCheckBox:Boolean read FMouseOnCheckBox write FMouseOnCheckBox;


  end;

var
  PSDBTreeViewCheckboxes: TPSDBTreeViewCheckboxes;

implementation

uses
  uPSDBTreeView.MC;

{%CLASSGROUP 'Vcl.Controls.TControl'}

{$R *.dfm}


procedure TPSDBTreeViewCheckboxes.ToggleSelectedCheckBox;
var TempNode:TTreeNode;
    startLevel: integer;
    mc:TPSDBTreeViewMC;
begin

mc:=(Self.Owner as TPSDBTreeViewMC);
if not FMouseOnCheckBox then exit;


with mc.mf.TreeView do begin

if Selected<>nil then begin

  if( Selected.StateIndex=2) then (Selected.StateIndex:=1) else
  if (Selected.StateIndex=1) then (Selected.StateIndex:=2);
  if (Selected.StateIndex=0) then ( Selected.StateIndex:=2);

  end;
// 1---if checked then checked for all children

if Selected.StateIndex=2 then
CheckNodeAndAllChildren(Selected) else
UnCheckNodeAndAllChildren(Selected);

// 2--- if unchecked any children then uncheck all parents



end;

end;


procedure TPSDBTreeViewCheckboxes.CheckNodeAndAllChildren(Node: TTreeNode);
var idParent:integer;
    i: Integer;
    mc:TPSDBTreeViewMC;
begin

mc:=(Self.Owner as TPSDBTreeViewMC);

if Assigned(Node) and Assigned(Node.Data) then
begin
idParent:=Integer(Node.Data^);
end else Exit;


//check all children in DB
SetLength(FIDArrayOfAllChildrenRecursive,0);
MC.Update.GatherIDofallChildrenRecursive(idParent,FIDArrayOfAllChildrenRecursive);


for i := 0 to High(FIDArrayOfAllChildrenRecursive) do begin

//  with (FMainFrame as TPSDBTreeExplorerMainFrame).MC.SQLQueries do begin
  with mc.SQL do begin
  SetUpdateIsCheckedWhereIDSQLText('1',FIDArrayOfAllChildrenRecursive[i].ToString);
  qUpdateIsCheckedWhereID.Connection:=mc.FDConnection;
  qUpdateIsCheckedWhereID.ExecSQL;

  end;

end;

//check node in db

//  with (FMainFrame as TPSDBTreeExplorerMainFrame).MC.SQLQueries do begin
  with MC.SQL do begin
  SetUpdateIsCheckedWhereIDSQLText('1',idParent.ToString);
  qUpdateIsCheckedWhereID.Connection:=mc.FDConnection;
  qUpdateIsCheckedWhereID.ExecSQL;
  end;


//Check Node and all visible-loaded from db children checkboxes
CheckNodeAndAllVisibleChildren(Node);


end;


procedure TPSDBTreeViewCheckboxes.UnCheckNodeAndAllChildren(Node: TTreeNode);
var idParent:integer;
  i: Integer;
  mc:TPSDBTreeViewMC;

begin

mc:=(Self.Owner as TPSDBTreeViewMC);

if Assigned(Node) and Assigned(Node.Data) then
begin
idParent:=Integer(Node.Data^);
end else Exit;


//uncheck all children in DB
SetLength(FIDArrayOfAllChildrenRecursive,0);


MC.Update.GatherIDofallChildrenRecursive(idParent,FIDArrayOfAllChildrenRecursive);


for i := 0 to High(FIDArrayOfAllChildrenRecursive) do begin
  with MC.SQL do begin
  SetUpdateIsCheckedWhereIDSQLText('0',FIDArrayOfAllChildrenRecursive[i].ToString);
  qUpdateIsCheckedWhereID.Connection:=mc.FDConnection;
  qUpdateIsCheckedWhereID.ExecSQL;
  end;
end;


//uncheck node in db

  with MC.SQL do begin
  SetUpdateIsCheckedWhereIDSQLText('0',idParent.ToString);
  qUpdateIsCheckedWhereID.Connection:=mc.FDConnection;
  qUpdateIsCheckedWhereID.ExecSQL;
  end;


//UnCheck Node and all visible-loaded from db children checkboxes
UnCheckNodeAndAllVisibleChildren(Node);


// Uncheck all parents in DB
GatherAllParentsIDAnduncheckInTree(Node);

for i := 0 to High(FIDArrayOfAllParents) do begin

  with MC.SQL do begin
  SetUpdateIsCheckedWhereIDSQLText('0',FIDArrayOfAllParents[i].ToString);
  qUpdateIsCheckedWhereID.Connection:=mc.FDConnection;
  qUpdateIsCheckedWhereID.ExecSQL;
  end;

end;



end;


procedure TPSDBTreeViewCheckboxes.GatherAllParentsIDAnduncheckInTree(Node: TTreeNode);
begin

SetLength(FIDArrayOfAllParents,0);

while Assigned(Node.Parent) do begin
Node:=Node.Parent;
SetLength(FIDArrayOfAllParents,Length(FIDArrayOfAllParents)+1);
FIDArrayOfAllParents[Length(FIDArrayOfAllParents)-1]:=Integer(Node.Data^);
Node.StateIndex:=1;
end;


end;


procedure TPSDBTreeViewCheckboxes.CheckNodeAndAllVisibleChildren(Node:TTreeNode);
var
  startLevel: integer;
  TempNode: TTreeNode;
  mc:TPSDBTreeViewMC;

begin

mc:=(Self.Owner as TPSDBTreeViewMC);

    with mc.mf.TreeView do begin

     Items.BeginUpdate;

        startLevel:=Node.Level;
        TempNode:=Node;
         repeat
         TempNode.StateIndex:=2; // set check box to true
         TempNode:=TempNode.GetNext;
         until (not Assigned(TempNode)) or (TempNode.Level<=StartLevel);


     Items.EndUpdate;

    end;


end;


constructor TPSDBTreeViewCheckboxes.Create(AOwner: TComponent);
begin
  inherited;

//  FMC:=(Self.Owner as TPSDBTreeViewMC)

end;

procedure TPSDBTreeViewCheckboxes.UnCheckNodeAndAllVisibleChildren(Node:TTreeNode);
var
  startLevel: integer;
  TempNode: TTreeNode;
  mc:TPSDBTreeViewMC;

begin

mc:=(Self.Owner as TPSDBTreeViewMC);

    with mc.mf.TreeView do begin

     Items.BeginUpdate;

        startLevel:=Node.Level;
        TempNode:=Node;
         repeat
         TempNode.StateIndex:=1; // set check box to true
         TempNode:=TempNode.GetNext;
         until (not Assigned(TempNode)) or (TempNode.Level<=StartLevel);


     Items.EndUpdate;

    end;


end;









procedure TPSDBTreeViewCheckboxes.DBTreeViewMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);

var mc:TPSDBTreeViewMC;

begin
//


mc:=(Self.Owner as TPSDBTreeViewMC);

with mc.MF.TreeView do begin

FNodeUnderMice:=nil;
if GetNodeAt(X,Y)<>nil then FNodeUnderMice:=GetNodeAt(X,Y);


    if FNodeUnderMice<>nil then begin

    if (X<FNodeUnderMice.DisplayRect(true).Left-16) and
    (X>FNodeUnderMice.DisplayRect(true).Left-32)
     then

        begin
        FMouseOnCheckBox:=true;
        //showmessage('Here');
        end

        else FMouseOnCheckBox:=False;

    end;



              //        ?
 if GetNodeAt(X,Y)<>nil then

 begin

 if (X<=FNodeUnderMice.DisplayRect(true).Left+FNodeUnderMice.DisplayRect(true).Width)

 then
 FUnderNode:=true else FUnderNode:=false

 end else FUnderNode:=false;




end;


end;



procedure TPSDBTreeViewCheckboxes.UncheckAllChecked;

var Node:TTreeNode;
    mc:TPSDBTreeViewMC;

begin

mc:=(Self.Owner as TPSDBTreeViewMC);

    with  mc.MF.TreeView do begin

     Node:=Items.GetFirstNode;
     if Node=nil then exit;

     repeat
     if Node.StateIndex=2 then
     UnCheckNodeAndAllChildren(Node);

     Node:=Node.GetNext;
     until not Assigned(Node) ;

    end;


end;



procedure TPSDBTreeViewCheckboxes.ClearAllCheckBoxesInDB;
var mc:TPSDBTreeViewMC;
begin

mc:=(Self.Owner as TPSDBTreeViewMC);

  with MC.SQL do begin
  SetClearAllCheckBoxesSQLText;
  qClearAllCheckBoxes.ExecSQL;

  end;


end;


end.
