unit uPSDBTreeView.Update;

interface

uses
  System.SysUtils, System.Classes, Vcl.ComCtrls;
//  uPSDBTreeView.MC;

type
  TPSDBTreeViewUpdate = class(TDataModule)
  private

    FIDArray:array of integer;
    FNamesArray:array of string;
    FIsCheckedArray:array of Boolean;
    FIDArrayChildrenOfChildren:array of integer;
    FUpdateOtherFieldsInTable:TNotifyEvent;

    procedure GatherIDofChildrenOfChildren(const idParent: integer);
    procedure GatherIDofallDirectChildrenOrdered(const idParent: integer);
    procedure GatherIDofallDirectChildren(const idParent: integer);


    { Private declarations }
  public
    { Public declarations }
   constructor Create(AOwner: TComponent); override;

   procedure UpdateNodeOnExpanding(Node: TTreeNode);
   procedure GatherIDofallChildrenRecursive(const idParent: integer; var AidArray: TArray<integer>);
   procedure ResortChildren;
   property  UpdateOtherFieldsInTable:TNotifyEvent read FUpdateOtherFieldsInTable write FUpdateOtherFieldsInTable;

//   property  MC:TPSDBTreeViewMC read FMC write FMC;

  end;

const SomeGhostNodeConst='{5071A2AB-05C0-4B7C-A233-CEF5C3089182}';
//var
//  PSDBTreeViewUpdate: TPSDBTreeViewUpdate;

implementation

uses uPSDBTreeView.Main,uPSDBTreeView.MC;
//var  Main_UV:TPSDBTreeView;

{%CLASSGROUP 'Vcl.Controls.TControl'}

{$R *.dfm}



procedure TPSDBTreeViewUpdate.UpdateNodeOnExpanding(Node:TTreeNode);
var
  i: Integer;
  idPointer:^integer;
  TrashPointer:^integer;
  idParent:Integer;
  AddedNode:TTreeNode;
  mc:TPSDBTreeViewMC;

begin

// link to owner mc - model controller
mc:=(Self.Owner as TPSDBTreeViewMC);

// getting idParent
if Node=nil then idParent:=-1 else
if Node.Data<>nil then idParent:=Integer(Node.Data^) else Exit;

//check if was expanded
if mc.isAlreadyExpandedSL.IndexOf(idParent.ToString)<>-1 then
Exit() else mc.isAlreadyExpandedSL.Add(idParent.ToString);

// getting id,name, isChecked of children
gatherIDofallDirectChildrenOrdered(idParent);

mc.MF.TreeView.Items.BeginUpdate;

// deleting ghost child
mc.Delete.ClearOnlyGhostChild(Node);


// loading data
 for i := 0 to High(FIDArray) do begin

 //
    if mc.isOnlyFoldersInDBTreeView then begin

     //if only folder
     if ( mc.CommonMethods.IsFolderByID(FIDArray[i]) )
     and not ( mc.CommonMethods.IfSuchChildExists(Node,FIDArray[i]) ) then begin

         // adding node
        New(idPointer);
        idPointer^:=(FIDArray[i]);
        AddedNode:=mc.MF.TreeView.Items.AddChildObject(Node,FNamesArray[i],idPointer);


        // checkBoxes
        if mc.MF.TreeView.StateImages<>nil then begin
        if FIsCheckedArray[i]=true then AddedNode.StateIndex:=2 else
        AddedNode.StateIndex:=1;
        end;


       // folder or file icon
       if mc.MF.TreeView.Images<>nil then begin  //AddedNode.ImageIndex:=1;
                 // folder or file icon
          if  MC.CommonMethods.IsFolderByID( integer(AddedNode.data^) ) then begin

          AddedNode.ImageIndex:=1;
          AddedNode.OverlayIndex:=1;
          AddedNode.SelectedIndex:=1;

          end else begin

          AddedNode.ImageIndex:=0;
          AddedNode.OverlayIndex:=0;
          AddedNode.SelectedIndex:=0;

          end;
       end;



        // addinng ghost nodes if node has children
        GatherIDofChildrenOfChildren(FIDArray[i]);
        if Length(FIDArrayChildrenOfChildren)>0 then
        mc.MF.TreeView.Items.AddChildObject(AddedNode,SomeGhostNodeConst,nil);



     end;

    end else
    // if files and folders
    if not ( mc.CommonMethods.IfSuchChildExists(Node,FIDArray[i]) ) then begin

         // adding node
        New(idPointer);
        idPointer^:=(FIDArray[i]);
        AddedNode:=mc.MF.TreeView.Items.AddChildObject(Node,FNamesArray[i],idPointer);


        // checkBoxes
        if mc.MF.TreeView.StateImages<>nil then begin
        if FIsCheckedArray[i]=true then AddedNode.StateIndex:=2 else
        AddedNode.StateIndex:=1;
        end;

       //check if FIDArray[i] has children and add ghost node
       if mc.MF.TreeView.Images<>nil then begin  //AddedNode.ImageIndex:=1;
                 // folder or file icon
          if  MC.CommonMethods.IsFolderByID( integer(AddedNode.data^) ) then begin

          AddedNode.ImageIndex:=1;
          AddedNode.OverlayIndex:=1;
          AddedNode.SelectedIndex:=1;

          end else begin

          AddedNode.ImageIndex:=0;
          AddedNode.OverlayIndex:=0;
          AddedNode.SelectedIndex:=0;

          end;
       end;

        // addinng ghost nodes if node has children
        GatherIDofChildrenOfChildren(FIDArray[i]);
        if Length(FIDArrayChildrenOfChildren)>0 then
        mc.MF.TreeView.Items.AddChildObject(AddedNode,SomeGhostNodeConst,nil);

    end;



 end;


mc.MF.TreeView.Items.EndUpdate;


end;












constructor TPSDBTreeViewUpdate.Create(AOwner: TComponent);
begin
  inherited;

// if (AOwner is TPSDBTreeView) then Main_UV:=(AOwner as TPSDBTreeView);

end;

procedure TPSDBTreeViewUpdate.GatherIDofallDirectChildrenOrdered(const idParent:integer);

var external_id_value:integer;
    mc:TPSDBTreeViewMC;

//    mainFrame:; //TPSDBTreeExplorerMainFrame;

begin

mc:=(Self.Owner as TPSDBTreeViewMC);

external_id_value:=mc.External_id_value;

// setting sql request in dependence of use external id
with mc do begin
 if  IsUseExternalID then
     SQL.SetSelectAllChildrenOrderedSQLText(idParent.ToString,'0',external_id_value)
 else
     SQL.SetSelectAllChildrenOrderedSQLText(idParent.ToString,'0');
end;

// getting data
 with mc.SQL.qSelectAllChildrenOrdered do begin

    Connection:=mc.FDConnection; //Main_UV.FDConnection;
    Open();
    SetLength(FIDArray,0);
    SetLength(FNamesArray,0);
    SetLength(FIsCheckedArray,0);

    while not EOF  do begin

    // gathering ID
    SetLength(FIDArray,Length(FIDArray)+1);
    FIDArray[Length(FIDArray)-1]:=FieldByName('id').AsInteger;

    // gethering names
    SetLength(FNamesArray,Length(FNamesArray)+1);
    FNamesArray[Length(FNamesArray)-1]:=FieldByName('name').AsString;

    // gathering isChecked
    SetLength(FIsCheckedArray,Length(FIsCheckedArray)+1);
    FIsCheckedArray[Length(FIsCheckedArray)-1]:=FieldByName('isChecked').AsBoolean;


    Next;
    end;

  Close();

 end;



end;



procedure TPSDBTreeViewUpdate.GatherIDofChildrenOfChildren(const idParent: integer);
var mc:TPSDBTreeViewMC;
begin

mc:=(Self.Owner as TPSDBTreeViewMC);

 MC.SQL.SetSelectAllChildrenOrderedSQLText(idParent.ToString,'0');

 with MC.SQL.qSelectAllChildrenOrdered do begin

  Open();
  SetLength(FIDArrayChildrenOfChildren,0);

  while not EOF  do begin
  SetLength(FIDArrayChildrenOfChildren,Length(FIDArrayChildrenOfChildren)+1);
  FIDArrayChildrenOfChildren[Length(FIDArrayChildrenOfChildren)-1]:=FieldByName('id').AsInteger;
  Next;
  end;

  Close();

 end;


end;



procedure TPSDBTreeViewUpdate.GatherIDofallChildrenRecursive(const idParent: integer;
  var AidArray: TArray<integer>);
var
  i: Integer;
  idArrayLocal:array of integer;
  mc:TPSDBTreeViewMC;

begin
//
mc:=(Self.Owner as TPSDBTreeViewMC);


 mc.SQL.SetSelectAllChildrenOrderedSQLText(idParent.ToString,'0');

   with MC.SQL.qSelectAllChildrenOrdered do begin

    Open();
    SetLength(idArrayLocal,0);

    while not EOF  do begin

    SetLength(idArrayLocal,Length(idArrayLocal)+1);
    idArrayLocal[Length(idArrayLocal)-1]:=FieldByName('id').AsInteger;

    Next;
    end;

    Close();

   end;

// transfer data to external array
for i := 0 to High(idArrayLocal) do begin
  SetLength(AidArray,Length(AidArray)+1);
  AidArray[Length(AidArray)-1]:=idArrayLocal[i];
end;

//recursion
for i := 0 to High(idArrayLocal) do GatherIDofallChildrenRecursive(idArrayLocal[i],AidArray);


end;



procedure TPSDBTreeViewUpdate.ResortChildren;
var
  i,idCurrentParent: Integer;
  mc:TPSDBTreeViewMC;

begin

mc:=(Self.Owner as TPSDBTreeViewMC);

//if not Assigned( mc.mf.TreeView.Selected ) then
//Exit;
//if not Assigned((FMainFrame as TPSDBTreeExplorerMainFrame).TreeView.Selected.data)
//then Exit;

if Assigned( mc.mf.TreeView.Selected )

then
idCurrentParent:=Integer (mc.mf.TreeView.Selected.Data^)
else
idCurrentParent:=-1;

//gather all children
GatherIDofallDirectChildren(idCurrentParent);

//update their indexes

  for i := Low(FIDArray) to High(FIDArray) do begin

  with MC do begin
  SQL.SetUpdateSortIndexSQLText(i.ToString,FIDArray[i].ToString());
  SQL.qUpdateSortIndex.Connection:=mc.FDConnection;
  SQL.qUpdateSortIndex.ExecSQL;
  end;
  end;

//showmessage('success');


end;



procedure TPSDBTreeViewUpdate.GatherIDofallDirectChildren(const idParent:integer);

var external_id_value:integer;
    mc:TPSDBTreeViewMC;


begin
//

mc:=(Self.Owner as TPSDBTreeViewMC);
external_id_value:=MC.External_id_value;

// setting sql request in dependence of use external id
with MC do begin
 if  IsUseExternalID then
 SQL.SetSelectAllChildrenSQLText(idParent.ToString,'0',external_id_value)
 else
 SQL.SetSelectAllChildrenSQLText(idParent.ToString,'0');
end;

// getting data
 with MC.SQL.qSelectAllChildren do begin

 Connection:=mc.FDConnection;

    Open();
    SetLength(FIDArray,0);
    SetLength(FNamesArray,0);
    SetLength(FIsCheckedArray,0);

    while not EOF  do begin

    // gathering ID
    SetLength(FIDArray,Length(FIDArray)+1);
    FIDArray[Length(FIDArray)-1]:=FieldByName('id').AsInteger;

    // gethering names
    SetLength(FNamesArray,Length(FNamesArray)+1);
    FNamesArray[Length(FNamesArray)-1]:=FieldByName('name').AsString;

    // gathering isChecked
    SetLength(FIsCheckedArray,Length(FIsCheckedArray)+1);
    FIsCheckedArray[Length(FIsCheckedArray)-1]:=FieldByName('isChecked').AsBoolean;


    Next;
    end;

  Close();

 end;



end;


end.
