unit uPSDBTreeView.SQL;

interface

uses
  System.SysUtils, System.Classes, FireDAC.Stan.Intf, FireDAC.Stan.Option, FireDAC.Stan.Param, FireDAC.Stan.Error,
  FireDAC.DatS, FireDAC.Phys.Intf, FireDAC.DApt.Intf, FireDAC.Stan.Async, FireDAC.DApt, Data.DB, FireDAC.Comp.DataSet,
  FireDAC.Comp.Client, FireDAC.UI.Intf, FireDAC.Stan.Def, FireDAC.Stan.Pool, FireDAC.Phys, FireDAC.Phys.MySQL,
  FireDAC.Phys.MySQLDef, FireDAC.VCLUI.Wait;

type
  TPSDBTreeViewSQL = class(TDataModule)
    qSelectAllChildrenOrdered: TFDQuery;
    qSelectWhereID: TFDQuery;
    qUpdateIsCheckedWhereID: TFDQuery;
    qDeleteWhereID: TFDQuery;
    qAddRecord: TFDQuery;
    qGetLastInsertedID: TFDQuery;
    qUpdateSortIndex: TFDQuery;
    qSelectAllChildren: TFDQuery;
    qInsertSelect: TFDQuery;
    qUpdateParent: TFDQuery;
    qUpdateExternalIDWhereID: TFDQuery;
    FDConnTemp: TFDConnection;
    FDStoredProc_copyRecord: TFDStoredProc;
    qInsertSelect2: TFDQuery;
    qClearAllCheckBoxes: TFDQuery;
    qUpdateNameWhereID: TFDQuery;
  private

    { Private declarations }
    FDBNamePlusDBTable:string;

    //short names for fields
    Fid_field:string;
    FidParent_field:string;
    FName_field:string;
    FSortIndex_field:string;
    FIsFolder_field:string;
    FIsTrash_field:string;
    FIsChecked_field:string;
    FExternal_id_field:string;




  public
    { Public declarations }

   constructor Create(AOwner: TComponent); override;

   procedure SetDBNamePlusDBTable;
   procedure SetShortNamesForFields;

   procedure SetSelectAllChildrenOrderedSQLText(const AidParentParam, AisTrashParam: string); overload;
   procedure SetSelectAllChildrenOrderedSQLText(const AidParentParam:string;const AisTrashParam:string; AExternal_id_value:integer); overload;
   procedure SetSelectAllChildrenSQLText(const AidParentParam, AisTrashParam: string); overload;
   procedure SetSelectAllChildrenSQLText(const AidParentParam, AisTrashParam: string; AExternal_id:Integer); overload;


   procedure SetUpdateIsCheckedWhereIDSQLText(const AisChecked, AidParam: string);
   procedure SetUpdateExternalIDWhereIDSQLText(const Aid:integer; const ANewExternalID:integer);

   procedure SetDeleteWhereIDSQLText(const AidParam: string);
   procedure SetAddRecordSQLText(const AidParentParam, AnameParam, AsortIndexParam, AisFolderParam,
      AisTrashParam: string); overload;
   procedure SetAddRecordSQLText(const AidParentParam, AnameParam, AsortIndexParam, AisFolderParam,
     AisTrashParam: string; const AExternal_id:integer); overload;
   procedure SetUpdateSortIndexSQLText(const AsortIndex, AidParam: string);

   procedure SetInsertSelectSQLText(const AidParam: string); overload;
   procedure SetInsertSelectSQLText(const AidParam: string; const AExternal_id:integer); overload;
   procedure SetInsertSelectSQLText2(AID:integer);
   procedure SetClearAllCheckBoxesSQLText;
   procedure SetUpdateNameWhereID(const AidParam: string; const AName:string);

   procedure SetUpdateParentSQLText(const AidParentNewParam, AidParam: string);

   function GetMaxID: integer;



   procedure SetSelectWhereIDSQLtext(const Aidparam: string);



    property id_field:string read Fid_field write Fid_field;
    property idParent_field:string read FidParent_field write FidParent_field;
    property Name_field:string read FName_field write FName_field;
    property External_id_field:string read FExternal_id_field write FExternal_id_field;

    property SortIndex_field:string read FSortIndex_field write FSortIndex_field;
    property IsFolder_field:string read FIsFolder_field write FIsFolder_field;
    property IsTrash_field:string read FIsTrash_field write FIsTrash_field;
    property IsChecked_field:string read FIsChecked_field write FIsChecked_field;



  end;

//var
//  PSDBTreeViewSQL: TPSDBTreeViewSQL;

implementation

{%CLASSGROUP 'Vcl.Controls.TControl'}

{$R *.dfm}

uses uPSDBTreeView.Main,uPSDBTreeView.MC;
var  Main_UV:TPSDBTreeView;


procedure TPSDBTreeViewSQL.
SetSelectAllChildrenOrderedSQLText(const AidParentParam:string;const AisTrashParam:string);

begin

// for example
//SELECT * FROM coffeetest_db.dbtreeexplorertesttable where idParent=-1 and isTrash=0 order by SortIndex;
qSelectAllChildrenOrdered.SQL.Text:=
'Select * from '+FDBNamePlusDBTable+
' where '+FidParent_field+'='+AidParentParam+' and '+
FIsTrash_field+'='+AisTrashParam+' order by '+FSortIndex_field;
;

end;


function TPSDBTreeViewSQL.GetMaxID:integer;
begin

//SELECT max(id) as newkey FROM coffeetest_db.dbtreeexplorertesttable;
//SELECT last_insert_id() as newkey;

qGetLastInsertedID.SQL.Text:=
//'Select max('+Fid_field+') as newkey from '+FDBNamePlusDBTable;
'select last_insert_id() as newkey';

with qGetLastInsertedID do begin
Open();
Result:=FieldByName('newkey').AsInteger;
Close();
end;

end;

procedure TPSDBTreeViewSQL.SetShortNamesForFields;
var mc:TPSDBTreeViewMC;
begin

mc:=(Self.Owner as TPSDBTreeViewMC);

    Fid_field:='`'+mc.DBFields.id+'`';
    FidParent_field:='`'+MC.DBFields.idParent+'`';
    FName_field:='`'+MC.DBFields.name+'`';
    FSortIndex_field:='`'+MC.DBFields.sortIndex+'`';
    FIsFolder_field:='`'+MC.DBFields.isFolder+'`';
    FIsTrash_field:='`'+MC.DBFields.isTrash+'`';
    FIsChecked_field:='`'+MC.DBFields.isChecked+'`';
    FExternal_id_field:='`'+MC.DBFields.external_id+'`';


end;



procedure TPSDBTreeViewSQL.SetSelectAllChildrenOrderedSQLText
(const AidParentParam:string;const AisTrashParam:string; AExternal_id_value:integer);

begin

// for example
//SELECT * FROM coffeetest_db.dbtreeexplorertesttable where idParent=-1 and isTrash=0 and external_id=1 order by SortIndex;

qSelectAllChildrenOrdered.SQL.Text:=
'Select * from '+FDBNamePlusDBTable+
' where '+FidParent_field+'='+AidParentParam+' and '+
FExternal_id_field+'='+AExternal_id_value.ToString+' and ' +
FIsTrash_field+'='+AisTrashParam+' order by '+FSortIndex_field;
;

end;




procedure TPSDBTreeViewSQL.SetSelectAllChildrenSQLText(const AidParentParam, AisTrashParam: string);
begin
//

// for example
//SELECT * FROM coffeetest_db.dbtreeexplorertesttable where idParent=-1 and isTrash=0 order by SortIndex;

qSelectAllChildren.SQL.Text:=
'Select * from '+FDBNamePlusDBTable+
' where '+FidParent_field+'='+AidParentParam+' and '+
FIsTrash_field+'='+AisTrashParam;
;


end;


procedure TPSDBTreeViewSQL.SetSelectAllChildrenSQLText(const AidParentParam, AisTrashParam: string; AExternal_id:Integer );
begin
//

// for example
//SELECT * FROM coffeetest_db.dbtreeexplorertesttable where idParent=-1 and isTrash=0 and External_id=1;

qSelectAllChildren.SQL.Text:=
'Select * from '+FDBNamePlusDBTable+
' where '+FidParent_field+'='+AidParentParam+' and '+
 FExternal_id_field+'='+AExternal_id.ToString+' and '+
FIsTrash_field+'='+AisTrashParam;
;


end;


procedure TPSDBTreeViewSQL.SetSelectWhereIDSQLtext(const Aidparam: string);
begin

//SELECT * FROM coffeetest_db.dbtreeexplorertesttable where id=95;
qSelectWhereID.SQL.Text:=
'Select * from '+FDBNamePlusDBTable+
' where '+Fid_field+'='+Aidparam;


end;

constructor TPSDBTreeViewSQL.Create(AOwner: TComponent);
begin
  inherited;

  if (AOwner is TPSDBTreeView) then Main_UV:=(AOwner as TPSDBTreeView);
//  FDConnTemp.Connected:=false;

end;

procedure TPSDBTreeViewSQL.SetDBNamePlusDBTable;
var mc:TPSDBTreeViewMC;
begin

mc:=(Self.Owner as TPSDBTreeViewMC);

with mc do begin
FDBNamePlusDBTable:='`'+DBName+'`'+'.'+'`'+DBTable+'`';
end;


end;


procedure TPSDBTreeViewSQL.SetUpdateExternalIDWhereIDSQLText(const Aid:integer; const ANewExternalID:integer);
begin

//UPDATE `coffeetest_db`.`dbtreeexplorertesttable` SET `external_id`='2' WHERE `id`='12198';
qUpdateExternalIDWhereID.SQL.Text:=
'SET foreign_key_checks = 0;'+
'UPDATE '+FDBNamePlusDBTable+
' SET '+FExternal_id_field+'='
+''''+ANewExternalID.ToString+''''
+' WHERE '+Fid_field+'='
+''''+Aid.ToString+''''+';'+'SET foreign_key_checks = 1;';


end;

procedure TPSDBTreeViewSQL.SetUpdateIsCheckedWhereIDSQLText(const AisChecked:string; const AidParam: string);
begin
//UPDATE `coffeetest_db`.`dbtreeexplorertesttable` SET `isChecked`='1' WHERE `id`='710';

qUpdateIsCheckedWhereID.SQL.Text:=
'SET foreign_key_checks = 0;'+
'UPDATE '+FDBNamePlusDBTable+
' SET '+FIsChecked_field+'='
+''''+AisChecked+''''
+' WHERE '+Fid_field+'='
+''''+AidParam+''''+';'+'SET foreign_key_checks = 1;';



end;


procedure TPSDBTreeViewSQL.SetDeleteWhereIDSQLText(const AidParam: string);
begin

//DELETE FROM `coffeetest_db`.`dbtreeexplorertesttable` WHERE `id`='100';

qDeleteWhereID.SQL.Text:=
'SET foreign_key_checks = 0;'+
'Delete  from '+FDBNamePlusDBTable+
' where '+Fid_field+'='+Aidparam+';'+'SET foreign_key_checks = 1;';


end;




procedure TPSDBTreeViewSQL.SetAddRecordSQLText(

                           const AidParentParam: string;
                           const AnameParam:string;
                           const AsortIndexParam:string;
                           const AisFolderParam:string;
                           const AisTrashParam:string


                           );


var s:string;

begin

//example
//INSERT INTO `coffeetest_db`.`dbtreeexplorertesttable` (`idParent`, `name`, `index`, `isFolder`, `isTrash`)
//VALUES ('1', '123', '0', '1', '0');

Assert(FDBNamePlusDBTable<>'');
Assert(FidParent_field<>'');
Assert(FName_field<>'');
Assert(FSortIndex_field<>'');
Assert(FIsFolder_field<>'');
Assert(FIsTrash_field<>'');



qAddRecord.SQL.Text:=
'SET foreign_key_checks = 0;'+
'INSERT INTO '+FDBNamePlusDBTable
+' ( '+FidParent_field+','+FName_field+','+FSortIndex_field+','+FIsFolder_field+','+FIsTrash_field+' ) '
+' VALUES ( '
+''''+AidParentParam+''','
+''''+AnameParam+''','
+''''+AsortIndexParam+''','
+''''+AisFolderParam+''','
+''''+AisTrashParam+''');'+'SET foreign_key_checks = 1;';;

//+//,:name,:index,:isFolder,:isTrash)';
//+' (VALUES :idParent,:name,:index,:isFolder,:isTrash)';
{
with qAddRecord.Params do begin

   ParamValues['idParent']:=idParentParam;
   ParamValues['name']:=nameParam;
   ParamValues['index']:=indexParam;
   ParamValues['isFolder']:=isFolderParam;
   ParamValues['isTrash']:=isTrashParam;


end;
}
end;





procedure TPSDBTreeViewSQL.SetAddRecordSQLText(

                           const AidParentParam: string;
                           const AnameParam:string;
                           const AsortIndexParam:string;
                           const AisFolderParam:string;
                           const AisTrashParam:string;
                           const AExternal_id:integer


                           );


var s:string;

begin

//example
//INSERT INTO `coffeetest_db`.`dbtreeexplorertesttable` (`idParent`, `name`, `index`, `isFolder`, `isTrash`,`external_id`)
//VALUES ('1', '123', '0', '1', '0');

Assert(FDBNamePlusDBTable<>'');
Assert(FidParent_field<>'');
Assert(FName_field<>'');
Assert(FSortIndex_field<>'');
Assert(FIsFolder_field<>'');
Assert(FIsTrash_field<>'');
Assert(FExternal_id_field<>'');




qAddRecord.SQL.Text:=

'SET foreign_key_checks = 0;'+
'INSERT INTO '+FDBNamePlusDBTable
+' ( '+FidParent_field+','+FName_field+','+FSortIndex_field+','+FIsFolder_field+','+FIsTrash_field+','+FExternal_id_field+' ) '
+' VALUES ( '
+''''+AidParentParam+''','
+''''+AnameParam+''','
+''''+AsortIndexParam+''','
+''''+AisFolderParam+''','
+''''+AisTrashParam+''','
+''''+AExternal_id.ToString+''');'+'SET foreign_key_checks = 1;';

//+//,:name,:index,:isFolder,:isTrash)';
//+' (VALUES :idParent,:name,:index,:isFolder,:isTrash)';
{
with qAddRecord.Params do begin

   ParamValues['idParent']:=idParentParam;
   ParamValues['name']:=nameParam;
   ParamValues['index']:=indexParam;
   ParamValues['isFolder']:=isFolderParam;
   ParamValues['isTrash']:=isTrashParam;


end;
}
end;


procedure TPSDBTreeViewSQL.SetClearAllCheckBoxesSQLText;
begin
//
//UPDATE `coffeetest_db`.`dbtreeexplorertesttable` SET `isChecked`='1' WHERE `id`='710';

qClearAllCheckBoxes.SQL.Text:=
'UPDATE '+FDBNamePlusDBTable+
' SET '+FIsChecked_field+'=0 '
+' WHERE '+Fid_field+'>-1';


end;

procedure TPSDBTreeViewSQL.
SetUpdateSortIndexSQLText(const AsortIndex:string; const AidParam: string);

begin
//example
//UPDATE `coffeetest_db`.`dbtreeexplorertesttable` SET `sortIndex`='1' WHERE `id`='4';

qUpdateSortIndex.SQL.Text:=
'SET foreign_key_checks = 0;'+
'UPDATE '+FDBNamePlusDBTable+
' SET '+FSortIndex_field+'='
+''''+AsortIndex+''''
+' WHERE '+Fid_field+'='
+''''+AidParam+''''+';'+'SET foreign_key_checks = 1;';

end;









procedure TPSDBTreeViewSQL.SetInsertSelectSQLText(const AidParam:string);
begin
//
{
  FCopyRecordInDB.Text := 'INSERT INTO  ' + DBNamePlusTable + '(' + idParent + ',' + NameField + ',' + IndexField + ',' + isChecked + ',' + IsExpanded + ',' + Level + ',' + IsFolder + ',' + TextContentField + ',' + BlobContentField + ',' + CreationDateTime + ',' + ChangesDateTime + ')' + //IsFolder+')'+
  ' SELECT ' + IdParent + ',' + NameField + ',' + IndexField + ',' + isChecked + ',' + IsExpanded + ',' + Level + ',' + IsFolder + ',' + TextContentField + ',' + BlobContentField + ',' + CreationDateTime + ',' + ChangesDateTime + ' from ' + DBNamePlusTable + ' where ' + idField + '=:id';

}
qInsertSelect.SQL.Text:='INSERT INTO  ' +FDBNamePlusDBTable+
'('+
FidParent_field+','+
FName_field+','+
FSortIndex_field+','+
FIsFolder_field+','+
FIsTrash_field+','+
FIsChecked_field+
')'+
' SELECT '+
//'('+
FidParent_field+','+
FName_field+','+
FSortIndex_field+','+
FIsFolder_field+','+
FIsTrash_field+','+
FIsChecked_field+
' FROM ' +FDBNamePlusDBTable+
' WHERE '+Fid_field+'='+AidParam;//+
//')';



end;


procedure TPSDBTreeViewSQL.SetInsertSelectSQLText(const AidParam:string; const AExternal_id:integer);
begin
//
{
  FCopyRecordInDB.Text := 'INSERT INTO  ' + DBNamePlusTable + '(' + idParent + ',' + NameField + ',' + IndexField + ',' + isChecked + ',' + IsExpanded + ',' + Level + ',' + IsFolder + ',' + TextContentField + ',' + BlobContentField + ',' + CreationDateTime + ',' + ChangesDateTime + ')' + //IsFolder+')'+
  ' SELECT ' + IdParent + ',' + NameField + ',' + IndexField + ',' + isChecked + ',' + IsExpanded + ',' + Level + ',' + IsFolder + ',' + TextContentField + ',' + BlobContentField + ',' + CreationDateTime + ',' + ChangesDateTime + ' from ' + DBNamePlusTable + ' where ' + idField + '=:id';

}
qInsertSelect.SQL.Text:='INSERT INTO  ' +FDBNamePlusDBTable+
'('+
FidParent_field+','+
FName_field+','+
FSortIndex_field+','+
FIsFolder_field+','+
FIsTrash_field+','+
FIsChecked_field+','+
FExternal_id_field+
')'+
' SELECT '+
//'('+
FidParent_field+','+
FName_field+','+
FSortIndex_field+','+
FIsFolder_field+','+
FIsTrash_field+','+
FIsChecked_field+','+
FExternal_id_field+
' FROM ' +FDBNamePlusDBTable+
' WHERE '+Fid_field+'='+AidParam;//+
//')';



end;



procedure TPSDBTreeViewSQL.SetInsertSelectSQLText2(AID: integer);
var mc:TPSDBTreeViewMC;
  withoutQuotes: string;
begin

mc:=(Self.Owner as TPSDBTreeViewMC);

withoutQuotes:=StringReplace(mc.DBTable,'`','',[rfReplaceAll]);

with qInsertSelect2 do begin
Connection:=mc.FDConnection;
params.ParamValues['id']:=AID;
params.ParamValues['tablename']:=withoutQuotes;//'dbtreeexplorertesttable';//FDBNamePlusDBTable;


end;

{
 with FDStoredProc_copyRecord do begin
 Connection:=mc.FDConnection;

  StoredProcName:='coffeetest_db.copyRecord';
  Prepare;
  Params[0].Value:=mc.DBTable;
  Params[1].Value:=AID;
//  ExecProc;

 end;
}

end;

procedure TPSDBTreeViewSQL.SetUpdateNameWhereID(const AidParam, AName: string);
begin
// UPDATE `coffeetest_db`.`dbtreeexplorertesttable` SET `name`='123' WHERE `id`='12490';


qUpdateNameWhereID.SQL.Text:=
'SET foreign_key_checks = 0;'+
'UPDATE '+FDBNamePlusDBTable+
' SET '+FName_field+'='
+''''+AName+''''
+' WHERE '+Fid_field+'='
+''''+AidParam+''''+';'+'SET foreign_key_checks = 1;';


end;

procedure TPSDBTreeViewSQL.SetUpdateParentSQLText(const AidParentNewParam, AidParam: string);
begin
//

qUpdateParent.SQL.Text:=
'SET foreign_key_checks = 0;'+
'UPDATE '+FDBNamePlusDBTable+
' SET '+FidParent_field+'='
+''''+AidParentNewParam+''''
+' WHERE '+Fid_field+'='
+''''+AidParam+''''+';'+'SET foreign_key_checks = 1;';



end;


end.
